#include <kapp.h>
#include <qxembed.h>
#include <qtimer.h>
#include <qstringlist.h>
#include <qdragobject.h>
#include <qobjectlist.h>
#include <kglobal.h>
#include <kconfig.h>
#include <kstddirs.h>
#include <kmessagebox.h>
#include <ksimpleconfig.h>
#include <kdesktopfile.h>
#include <dcopclient.h>
#include "appletarea.h"
#include "panel.h"

extern Panel *panel;

template class QList<AppletContainer>;

AppletArea::AppletArea( Orientation orientation, int strutext,
		    QWidget* parent, const char* name)
    : Panner( orientation, strutext, parent, name ), DCOPObject("appletArea")
{
    KConfig* config = KGlobal::config();
    config->setGroup("applets");
    QStringList commands =  config->readListEntry( "commands" );
    if (commands.isEmpty()){
        commands.append("kminipagerapplet");
        commands.append("kdockapplet");
        commands.append("ktaskbarapplet");
        commands.append("kdateapplet");
    }

    QStringList desktopFiles = config->readListEntry("internallink");
    if(desktopFiles.isEmpty()){ // default desktop files for plugins
        desktopFiles.append("ktaskbarapplet.desktop");
        desktopFiles.append("kminipagerapplet.desktop");
        desktopFiles.append("kdockapplet.desktop");
        desktopFiles.append("kdateapplet.desktop");
        config->writeEntry("internallink", desktopFiles);
        config->sync();
    }

    int appw = orientation == Horizontal ? panel->buttonSize() :
        panel->width()-2*(panel->frameWidth()+frameWidth());
    int apph = orientation == Horizontal ? panel->height()-2*(panel->frameWidth()+frameWidth()):
        panel->buttonSize();

    QStringList::Iterator it;
    InternalFrame *frame;
    for ( it = commands.begin(); it != commands.end(); ++it ) {

        if (*it == "none") break;

        // some fixed stuff applets
        else if (*it == "ktaskbarapplet") {
            frame = new InternalFrame("ktaskbarapplet",
                                      desktopFiles[0], viewport());
            frame->setupGeometry( orientation, appw, apph);
            applets.append(frame);
        }
        else if (*it == "kminipagerapplet") {
            frame = new InternalFrame("kminipagerapplet",
                                      desktopFiles[1], viewport());
            frame->setupGeometry( orientation, appw, apph);
            applets.append(frame);
        }
        else if (*it == "kdockapplet") {
            frame = new InternalFrame("kdockapplet",
                                      desktopFiles[2], viewport());
            frame->setupGeometry( orientation, appw, apph);
            applets.append(frame);
        }
        else if (*it == "kdateapplet") {
            frame = new InternalFrame("kdateapplet",
                                      desktopFiles[3], viewport());
            frame->setupGeometry( orientation, appw, apph);
            applets.append(frame);
        }
        else{
            KProcess process;
            process << *it;
            process.start(KProcess::DontCare);
        }
    }
    writeCommands();
}

void AppletArea::setOrientation(Orientation orientation)
{
    Panner::setOrientation( orientation );
    int appw = orientation == Horizontal ? panel->buttonSize() :
        panel->width()-2*(panel->frameWidth()+frameWidth());
    int apph = orientation == Horizontal ? panel->height()-2*(panel->frameWidth()+frameWidth()):
        panel->buttonSize();

    for ( QListIterator<AppletContainer> it(applets); it.current(); ++it )
    {
	it.current()->setupGeometry( orientation, appw, apph );
    }
    layoutChildren(); //CT this has to do nice things
}

// internal, handles new applets
void AppletArea::dockMe( QCString application, QCString applet )
{
    AppletFrame* frame = new AppletFrame( application, applet, viewport() );
    applets.append( frame );
    connect( frame, SIGNAL( embeddedWindowDestroyed() ), this, SLOT( embeddedWindowDestroyed() ) );
    frame->setupGeometry( orientation(), panel->buttonSize(), panel->buttonSize());
    frame->show();
    layoutChildren();
    writeCommands();
}

void AppletArea::moveMe( QCString applet )
{
    AppletContainer* a= findApplet( applet );
    // do something with a! ### TODO
	
    KMessageBox::sorry( this, "Moving not yet supported");
}

void AppletArea::removeMe( QCString applet )
{
    AppletContainer* a= findApplet( applet );
    applets.removeRef( a );
    delete a;
    layoutChildren();
    writeCommands();
}

bool AppletArea::process(const QCString &fun, const QByteArray &data,
	     QCString& replyType, QByteArray &replyData)
{
    if ( fun == "dockMe(QCString)" ) {
	QDataStream dataStream( data, IO_ReadOnly );
	QCString name;
	dataStream >> name;
	dockMe( kapp->dcopClient()->senderId(), name );
	return TRUE;
    } else if ( fun == "moveMe(QCString)" ) {
	QDataStream dataStream( data, IO_ReadOnly );
	QCString name;
	dataStream >> name;
	moveMe( name );
	return TRUE;
    } else if ( fun == "removeMe(QCString)" ) {
	QDataStream dataStream( data, IO_ReadOnly );
	QCString name;
	dataStream >> name;
	removeMe( name );
	return TRUE;
    } else if(fun == "position()"){
        QDataStream dataStream(replyData, IO_ReadWrite);
        replyType = "int";
        dataStream << (int)panel->position();
        return(true);
    }
    else if(fun == "setStretch(QCString,int)"){
        QDataStream dataStream(data, IO_ReadOnly);
        QCString name;
        int s;
        dataStream >> name;
        dataStream >> s;
        AppletContainer* a = findApplet(name);
        if(a)
            a->setStretch(s);
        return(true);
    }
    return FALSE;
}

AppletArea::~AppletArea()
{
    writeCommands();
}

void AppletArea::writeCommands()
{
    QStringList commands;
    for ( AppletContainer* a = applets.first(); a; a = applets.next() ) {
        if(a->command() != QString::null)
            commands += a->command();
    }

    //CT perhaps user really doesn't want any applet - needed in order to
    //  avoid default fixed applets load at next startup
    if (commands.isEmpty()) commands.append("none");

    KConfig* config = KGlobal::config();
    config->setGroup("applets");
    config->writeEntry( "commands", commands );
    config->sync();

}

void AppletArea::layoutChildren()
{
    static QBoxLayout  *layout = NULL;
    static Orientation  oldOrient;
    static unsigned int oldAppletCount;
    int stretchUsed = 0;

    if(layout && oldOrient == orientation() &&
       applets.count() == oldAppletCount)
        return;

    oldAppletCount = applets.count();
    oldOrient = orientation();

    delete layout;
    if(orientation() == Horizontal)
        layout = new QHBoxLayout(viewport());
    else
        layout = new QVBoxLayout(viewport());

    QObjectListIt it( *viewport()->children() );
    QObject *obj;
    while ((obj=it.current())) {
        ++it;
        if (obj->isWidgetType()){
            if(obj->inherits("AppletContainer")){
                layout->addWidget((QWidget *)obj,
                                  ((AppletContainer *)obj)->stretch());
                stretchUsed += ((AppletContainer *)obj)->stretch();
            }
            else
                layout->addWidget((QWidget *)obj);

        }
    }
    if(!stretchUsed)
        layout->addStretch(1);

    resizeContents(layout->sizeHint().width(),
                   layout->sizeHint().height());

}

AppletContainer* AppletArea::findApplet( QCString id )
{
    for ( QListIterator<AppletContainer> it(applets); it.current(); ++it ) {
        if ( it.current()->objId() == id )
            return it.current();
    }
    return 0;
}


void AppletArea::embeddedWindowDestroyed()
{
    if ( sender() ) {
        ( (QWidget*) sender() )->close();
        applets.remove( (AppletFrame*)sender() );
    }

    QTimer::singleShot( 0, this, SLOT( cleanupChildren() ) );
}


AppletFrame::AppletFrame( QCString application, QCString objId, QWidget* parent )
    : AppletContainer(parent)
{
    app = application;
    obj = objId;

    // get some information about the applet.

    DCOPClient* dcop = kapp->dcopClient();
    win = 0;

    QByteArray data;
    QCString replyType;
    QByteArray replyData;
    if ( dcop->call ( application, obj, "winId()", data, replyType, replyData ) )
    {
	QDataStream reply( replyData, IO_ReadOnly );
	reply >> win;
    }
    if ( dcop->call ( application, obj, "restartCommand()", data, replyType, replyData ) )
    {
	QDataStream reply( replyData, IO_ReadOnly );
        reply >> com;
        if(com.isEmpty())
            com = app;
    }

    if ( win ) {
	QXEmbed* embed = new QXEmbed( this );
	embed->embed( win );
	connect ( embed, SIGNAL( embeddedWindowDestroyed() ), this, SIGNAL( embeddedWindowDestroyed() ) );
    }
}

void AppletFrame::setupGeometry( Orientation orientation, int width, int height )
{

    DCOPClient* dcop = kapp->dcopClient();
    QByteArray data;
    QCString replyType;
    QByteArray replyData;

    QDataStream dataStream( data, IO_WriteOnly );
    dataStream << int( orientation ) << width <<  height;

    resize( width, height );

    // call to be synchronous
    dcop->call ( app, obj, "setupGeometry(int,int,int)", data, replyType, replyData );

    layout()->activate();
    updateGeometry();
}

InternalFrame::InternalFrame(const QString &aname, const QString &desktopFile,
                             QWidget *parent)
    : AppletContainer(parent)
{
    applet = panel->pluginMgr()->loadPlugin(desktopFile, this);
    id = aname; // good as anything else ;-)
    if(applet)
    {
        connect(applet, SIGNAL(internalRemove()), SLOT(slotRemove()));
        connect(applet, SIGNAL(internalMove()), SLOT(slotMove()));
        stretchSize = applet->stretch();
    }
}

void InternalFrame::setupGeometry(Orientation orientation, int width,
                                  int height )
{
    resize(width, height);
    if(applet)
    {
        applet->setPosition((InternalApplet::Position)panel->position());
        applet->setupGeometry(orientation, width, height);
    }
    layout()->activate();
    updateGeometry();
}

QCString InternalFrame::objId() const
{
    return(id);
}

QString InternalFrame::command() const
{
    return(id);
}

void InternalFrame::slotRemove()
{
    ;
}

void InternalFrame::slotMove()
{
    ;
}



