#ifndef APPLETAREA_H
#define APPLETAREA_H

#include <qvbox.h>
#include <kprocess.h>
#include <qlist.h>
#include <dcopobject.h>
#include <kapplet.h>
#include "internalapplet.h"
#include "pluginmgr.h"
#include "panner.h"

class QXEmbed;

class AppletContainer;
class AppletFrame;

class AppletArea : public Panner, DCOPObject
{
    Q_OBJECT
public:
    AppletArea( Orientation orientation, int strut, QWidget* parent, const char* name = 0);
    ~AppletArea();
    
    void setOrientation(Orientation orientation);

    bool process(const QCString &fun, const QByteArray &data,
                 QCString& replyType, QByteArray &replyData);
protected:
    virtual void layoutChildren();
    void writeCommands();

private slots:
    void embeddedWindowDestroyed();

private:
    void dockMe( QCString application, QCString applet); // handles new applets
    void moveMe( QCString applet );
    void removeMe( QCString applet );
    
    AppletContainer* findApplet( QCString id );
    QList<AppletContainer> applets;
};

// so we can have a common pointer to both internal applets and applet frames
class AppletContainer : public QVBox
{
    Q_OBJECT
public:
    AppletContainer(QWidget *parent=0)
        : QVBox(parent, "AppletContainter")
            {setFrameStyle(NoFrame); stretchSize = 0;}
    virtual void setupGeometry(Orientation orientation, int width, int height)=0;
    virtual QCString objId() const=0;
    virtual QString command() const=0;
    virtual int stretch(){return(stretchSize);}
    virtual void setStretch(int s){stretchSize = s;}
protected:
    int stretchSize;
};

// internal shared lib applets
class InternalFrame : public AppletContainer
{
    Q_OBJECT
public:
    InternalFrame(const QString &aname, const QString &desktopFile,
                  QWidget *parent);
    void setupGeometry( Orientation orientation, int width, int height );
    QCString objId() const;
    QString command() const;
protected slots:
    void slotRemove();
    void slotMove();
protected:
    QCString id;
    InternalApplet *applet;
};
    
// external applets
class AppletFrame : public AppletContainer
{
    Q_OBJECT
public:
    AppletFrame(QCString application, QCString objId, QWidget* parent );
    QString command() const;
    WId window() const;
    QCString objId() const;
    void setupGeometry( Orientation orientation, int width, int height );
signals:
    void embeddedWindowDestroyed();
private:
    KProcess process;
    QCString app,obj,com;
    WId win;
};

inline QString AppletFrame::command() const
{
    return com;
}

inline QCString AppletFrame::objId() const
{
    return obj;
}

inline WId AppletFrame::window() const
{
    return win;
}

#endif

