/*
 * (C) Daniel M. Duley (mosfet@kde.org) KDE artistic license.
 * (C) Matthias Ettrich <ettrich@kde.org> KDE Artistic.
 * (C) Preston Brown <pbrown@kde.org> KDE artistic license.
 */

#include <qpixmap.h>
#include <dcopclient.h>
#include "clientmenu.h"
#include "panelbutton.h"

template class QList<KickerClientMenu>;

KickerClientMenu::KickerClientMenu( QWidget * parent, const char *name )
    : QPopupMenu( parent, name), DCOPObject( name )
{
}

KickerClientMenu::~KickerClientMenu()
{
}

void KickerClientMenu::clear()
{
    QPopupMenu::clear();
}
void KickerClientMenu::insertItem( QPixmap icon, QString text, int id )
{
    int globalid = QPopupMenu::insertItem( icon, text, this, SLOT( slotActivated(int) ) );
    setItemParameter( globalid, id );
}

void KickerClientMenu::insertItem( QString text, int id )
{
    int globalid = QPopupMenu::insertItem( text, this, SLOT( slotActivated(int) ) );
    setItemParameter( globalid, id );
}

void KickerClientMenu::connectDCOPSignal( QCString signal, QCString appId, QCString objId )
{
    // very primitive right now
    if ( signal == "activated(int)" ) {
	app = appId;
	obj = objId;
    } else {
	qWarning("DCOP: no such signal %s::%s", className(), signal.data() );
    }
}

bool KickerClientMenu::process(const QCString &fun, const QByteArray &data,
			       QCString &replyType, QByteArray &/*replyData*/)
{
    if ( fun == "clear()" ) {
	clear();
	replyType = "void";
	return TRUE;
    }
    else if ( fun == "insertItem(QPixmap,QString,int)" ) {
	QDataStream dataStream( data, IO_ReadOnly );
	QPixmap icon;
	QString text;
	int id;
	dataStream >> icon >> text >> id;
	insertItem( icon, text, id );
	replyType = "void";
	return TRUE;
    }
    else if ( fun == "insertItem(QString,int)" ) {
	QDataStream dataStream( data, IO_ReadOnly );
	QString text;
	int id;
	dataStream >> text >> id;
	insertItem( text, id );
	replyType = "void";
	return TRUE;
    }
    else if ( fun == "connectDCOPSignal(QCString,QCString,QCString)" ) {
	QDataStream dataStream( data, IO_ReadOnly );
	QCString signal, appId, objId;
	dataStream >> signal >> appId >> objId;
	connectDCOPSignal( signal, appId, objId );
	replyType = "void";
	return TRUE;
    }
    return FALSE;
}

void KickerClientMenu::slotActivated(int id)
{
    if ( !app.isEmpty()  ) {
	QByteArray data;
	QDataStream dataStream( data, IO_WriteOnly );
	dataStream << id;
	kapp->dcopClient()->send( app, obj, "activated(int)", data );
    }
}

///////////////////////////////////////////////////////////////////////////////


KickerMenuManager::KickerMenuManager( PanelMenu* menu, QObject *parent, const char *name )
    : QObject( parent, name ), DCOPObject( name )
{
    panelmenu = menu;
    clientmenus.setAutoDelete( TRUE );
    connect( kapp->dcopClient(), SIGNAL( applicationRemoved(const QCString&) ),
	     this, SLOT( applicationRemoved( const QCString& ) ) );
}

KickerMenuManager::~KickerMenuManager()
{
}

static int menucount = 0;
QCString KickerMenuManager::createMenu( QPixmap icon, QString text )
{
    menucount++;
    QCString name;
    name.sprintf("kickerclientmenu-%d", menucount );
    KickerClientMenu* p = new KickerClientMenu( 0, name );
    clientmenus.append(p);
    panelmenu->initialize();
    p->idInParentMenu = panelmenu->insertItem( icon, text, (QPopupMenu*)p );
    p->createdBy = kapp->dcopClient()->senderId();
    panelmenu->adjustSize();
    return name;
}

void KickerMenuManager::removeMenu( QCString menu )
{
    QListIterator<KickerClientMenu> it( clientmenus);
    KickerClientMenu* m;
    while ( ( m = it.current() ) ) {
	++it;
	if ( m->objId() == menu ) {
	    panelmenu->removeItem( m->idInParentMenu );
	    clientmenus.removeRef( m );
	}
    }
    panelmenu->adjustSize();
}


void KickerMenuManager::applicationRemoved( const QCString& appRemoved)
{
    QListIterator<KickerClientMenu> it( clientmenus);
    KickerClientMenu* m;
    while ( ( m = it.current() ) ) {
	++it;
	if ( m->createdBy == appRemoved ) {
	    panelmenu->removeItem( m->idInParentMenu );
	    clientmenus.removeRef( m );
	}
    }
    panelmenu->adjustSize();
}

bool KickerMenuManager::process(const QCString &fun, const QByteArray &data,
				QCString &replyType, QByteArray &replyData)
{
    if ( fun == "createMenu(QPixmap,QString)" ) {
	QDataStream dataStream( data, IO_ReadOnly );
	QPixmap icon;
	QString text;
	dataStream >> icon >> text;
	QDataStream reply( replyData, IO_WriteOnly );
	reply << createMenu( icon, text );
	replyType = "QCString";
	return TRUE;
    } else if ( fun == "removeMenu(QCString)" ) {
	QDataStream dataStream( data, IO_ReadOnly );
	QCString menu;
	dataStream >> menu;
	removeMenu( menu );
	replyType = "void";
	return TRUE;
    }
    else if( fun == "popupKMenu(int,int)" ){
        QDataStream dataStream( data, IO_ReadOnly );
        int x, y;
        dataStream >> x;
        dataStream >> y;
        emit popupKMenu(x, y);
        replyType = "void";
        return(true);
    }
    return FALSE;
}

#include "clientmenu.moc"

