#ifndef __CLIENTMENU_H
#define __CLIENTMENU_H

/*
 * (C) Daniel M. Duley (mosfet@kde.org) KDE artistic license.
 * (C) Matthias Ettrich <ettrich@kde.org> KDE Artistic.
 * (C) Preston Brown <pbrown@kde.org> KDE artistic license.
 */

#include <qlist.h>
#include <qstringlist.h>
#include <qpopupmenu.h>

#include <dcopobject.h>
#include <kapp.h>


class PanelMenu;

// Classes to handle client application menus. Used by PanelKButton, which
// also manages the toplevel K Button Menu.

/**
 * Small additions to QPopupMenu to contain data we need for DCop handling
 */
class KickerClientMenu : public QPopupMenu, DCOPObject
{
    Q_OBJECT
public:
    KickerClientMenu( QWidget *parent=0, const char *name=0);
    ~KickerClientMenu();

    // dcop exported
    void clear();
    void insertItem( QPixmap icon, QString text, int id );
    void insertItem( QString text, int id );
    // dcop signals:
    //     void activated(int)

    void connectDCOPSignal( QCString signal, QCString appId, QCString objId );

    // dcop internal
    virtual bool process(const QCString &fun, const QByteArray &data,
			 QCString &replyType, QByteArray &reply);

protected slots:
    void slotActivated(int id);

private:
    QCString app, obj; // for the signal

    // for the KickerClientMenu, internal
    friend class KickerMenuManager;
    int idInParentMenu;
    QCString createdBy;
};

/**
 * The factory for menus created by other applications. Also the owner of these menus.
 */
class KickerMenuManager : public QObject, DCOPObject
{
    Q_OBJECT
public:
    KickerMenuManager( PanelMenu* menu, QObject *parent=0, const char *name=0 );
    ~KickerMenuManager();

    // dcop exported

    QCString createMenu( QPixmap icon, QString text );
    void removeMenu( QCString menu );

    // dcop internal
    bool process(const QCString &fun, const QByteArray &data, QCString& replyType, QByteArray &reply);


protected slots:
    void applicationRemoved( const QCString& );
signals:
    void popupKMenu(int x, int y);
protected:
    PanelMenu* panelmenu;
    QList<KickerClientMenu> clientmenus;
};

#endif
