#include <qtimer.h>
#include <qtooltip.h>
#include <qlabel.h>
#include <qpopupmenu.h>
#include <qdatetime.h>
#include <qtimer.h>
#include <time.h>

#include <kglobal.h>
#include <kprocess.h>
#include <klocale.h>
#include <kstddirs.h>
#include <kdialog.h>
#include <kwinmodule.h>

#include "dateapplet.h"

KWinModule *kwin_module;

extern "C"
{
    InternalApplet* init(QWidget *parent, KWinModule *module)
    {
        kwin_module = module;
        return new DateApplet(parent, "kdateapplet");
    }
}

class ClockLabel : public QLabel
{
public:
  ClockLabel(QWidget *parent = 0, const char *name = 0)
    : QLabel(parent, name) 
    {
      rmbMenu = new QPopupMenu(this);
      rmbMenu->insertItem(i18n("Change Date/Time"));
    }
 void mousePressEvent(QMouseEvent *e)
    {
      if (e->button() == RightButton) {
	if (!KStandardDirs::findExe("kcmtime"))
	  return;
	else {
	  int i = rmbMenu->exec(this->mapToGlobal(QPoint(0-rmbMenu->sizeHint().width(), 0)));
	  i = rmbMenu->indexOf(i);
	  if (i == 0) {
	    KProcess proc;
	    proc << KStandardDirs::findExe("kcmtime");
	    proc.start(KProcess::DontCare);
	  }
	}
      }
    }

private:
  QPopupMenu *rmbMenu;
};

DateApplet::DateApplet(QWidget* parent, const char* name )
    : InternalApplet( parent, name )
{
    label = new ClockLabel( this );

    // if the time configuration program is available, make a menu
    // saying so
    
    
    // for now, should be configurable ####
    clockBeats = FALSE;

    QTime cur_time = QTime::currentTime();
    clock_timer = 0;
    int next_minute_break = (60 - cur_time.second()) * 1000 - cur_time.msec();
    QTimer::singleShot(next_minute_break, this, SLOT(updateClock()));
}

DateApplet::~DateApplet()
{
}

void DateApplet::setupGeometry( Orientation orientation, int width, int height )
{
  orient = orientation;

  // fix label_date size (PGB)
  int maxSize = 0, strWidth;
  QDateTime datetime(QDateTime::currentDateTime());
  QString longStr;

  for (int i = 0; i < 6; i++) {
    strWidth = label->fontMetrics().width(KGlobal::locale()->WeekDayName(datetime.addDays(i).date().dayOfWeek()) + "\n");
    if (strWidth > maxSize) {
      maxSize = strWidth;
      longStr = KGlobal::locale()->WeekDayName(datetime.addDays(i).date().dayOfWeek()) + "\n";
    }
  }

  strWidth = label->fontMetrics().width(KGlobal::locale()->formatDate(datetime.date(), true));
  if (strWidth > maxSize) {
    maxSize = strWidth;
    longStr = KGlobal::locale()->formatDate(datetime.date(), true);
  } 

  strWidth = label->fontMetrics().width(KGlobal::locale()->formatTime(datetime.time()));
  if (strWidth > maxSize) {
    maxSize = strWidth;
    longStr = KGlobal::locale()->formatTime(datetime.time());
  }
  
  QFont tmpFont(label->font());
  tmpFont.setPointSize(10);
  label->setFont(tmpFont);

  if (orient == Horizontal) {
    while (label->fontMetrics().lineSpacing() + 
	   KDialog::marginHint() * 2 &&
	   tmpFont.pointSize() >= 10) {
      tmpFont.setPointSize(tmpFont.pointSize()-1);
      if (tmpFont.exactMatch())
	label->setFont(tmpFont);
    }
  } else {
    while (label->fontMetrics().boundingRect(longStr).width() +
	   KDialog::marginHint() * 2 > width &&
	   tmpFont.pointSize() >= 8) {
      tmpFont.setPointSize(tmpFont.pointSize()-1);
      if (tmpFont.exactMatch())
	label->setFont(tmpFont);
    }
  }

  setMinimumSize(label->fontMetrics().boundingRect(longStr).width() + 
		 ((orient == Horizontal) ? KDialog::marginHint() * 2 : 
		  KDialog::marginHint()),
		 label->fontMetrics().lineSpacing() + 
		 ((orient == Horizontal) ? KDialog::marginHint() * 2 :
		  KDialog::marginHint()));  
  setLabelDate();
}


void DateApplet::resizeEvent( QResizeEvent* )
{
  label->setGeometry( rect() );
}


void DateApplet::setLabelDate()
{
  QDateTime datetime = QDateTime::currentDateTime();

  QString dayline;
  QString timeline;
  QString dateline;

  dayline = KGlobal::locale()->WeekDayName(datetime.date().dayOfWeek());

  if (orient == Vertical)
    dayline.truncate(3); // only keep first three letters

  dayline += "\n";

  if (clockBeats) {
    time_t curtime = time(0);
    struct tm *loctime = gmtime(&curtime);
    long iTime = (((loctime->tm_hour*3600 + 
		    loctime->tm_min*60 + 
		    loctime->tm_sec)+3600)*1000)/86400;

    if( iTime >= 1000 )
      iTime -= 1000;
    else if( iTime < 0 )
      iTime += 1000;

    timeline.sprintf("@%.3d", (int)iTime );
  } else {
    timeline = KGlobal::locale()->formatTime(datetime.time());
  }

  dateline = "\n" + KGlobal::locale()->formatDate(datetime.date(), true);

  if (label->fontMetrics().lineSpacing() * 3 <= height()) {
    label->setText(QString(dayline)+timeline+dateline);
    QToolTip::remove(label);
  } else if (fontMetrics().lineSpacing() * 2 <= height()) {
    label->setText(timeline+dateline);
    QToolTip::remove(label);
  } else {
    label->setText(timeline);
    QToolTip::add(label, KGlobal::locale()->formatDate(datetime.date()));
  }
}

void DateApplet::updateClock()
{
  setLabelDate();
  
  if ( !clock_timer ) {
    clock_timer = new QTimer(this);
    connect(clock_timer, SIGNAL(timeout()),
	    this, SLOT( updateClock() ) );
    (void) clock_timer->start(clockBeats ? 86400 : 60000);
  }
  // Adjust time interval in case system time has been adjusted
  // (i.e. ntpdate or something similiar)
  
  QTime cur_time = QTime::currentTime();
  
  if(clockBeats) {
    struct tm* loctime;
    time_t curtime = time(0);
    loctime = gmtime(&curtime);
    long next_break = 86400-((((loctime->tm_hour*3600 + loctime->tm_min*60 +
				loctime->tm_sec)+3600)*1000)%86400);
    clock_timer->changeInterval(next_break);
  } else
    clock_timer->changeInterval((60 - cur_time.second()) * 1000 -
				cur_time.msec());
}
