#include <../../panel.h>
#include <qpainter.h>
#include <kwinmodule.h>
#include <X11/Xlib.h>

KWinModule* kwin_module;

#include "dockapplet.h"

extern "C"
{
    InternalApplet* init(QWidget *parent, KWinModule *module)
    {
        kwin_module = module;
        return new DockAreaApplet(parent, "kdockapplet");
    }
}


KXEmbed::KXEmbed(QWidget *parent, const char *name, WFlags f)
    : QXEmbed(parent, name, f)
{}

KXEmbed::~KXEmbed()
{
    // This prevents ~QXEmbed from destroying the window.
    WId id = embeddedWinId();
    XEvent e;
    e.type = DestroyNotify;
    e.xdestroywindow.window = id;
    QXEmbed::x11Event(&e);

    XReparentWindow(qt_xdisplay(), id, qt_xrootwin(), 0, 0);
}
    

DockAreaApplet::DockAreaApplet( QWidget* parent, const char* name )
    : InternalApplet( parent, name )
{
    connect( kwin_module, SIGNAL( dockWindowAdd(WId) ), this, SLOT( dockWindowAdd(WId) ) );
    connect( kwin_module, SIGNAL( dockWindowRemove(WId) ), this, SLOT( updateDockWindows() ) );

    want_width = want_height = -1;
    m_Docks.setAutoDelete(true);
}


DockAreaApplet::~DockAreaApplet()
{
    m_Docks.clear();
}

void DockAreaApplet::paintEvent( QPaintEvent* )
{
    QPainter p( this );
    style().drawPanel( &p, 0, 0, width(), height(), colorGroup(), TRUE );
}

void DockAreaApplet::setupGeometry( Orientation, int width, int height )
{
    want_width = width;
    want_height = height;
}

void DockAreaApplet::resizeEvent( QResizeEvent* )
{
    layoutDockingArea();
}

QSize DockAreaApplet::sizeHint() const
{
    return QSize( want_width, want_height );
}

void DockAreaApplet::dockWindowAdd( WId w )
{
    KXEmbed *emb = new KXEmbed(this);
    connect(emb, SIGNAL(embeddedWindowDestroyed()), SLOT(updateDockWindows()));
    m_Docks.append(emb);

    emb->embed(w);
    emb->resize(24, 24);
    emb->show();

    layoutDockingArea();
}

void DockAreaApplet::updateDockWindows()
{
    KXEmbed *emb = m_Docks.first();
    while ((emb = m_Docks.current()) != 0L) {
	WId wid = emb->embeddedWinId();
	if ((wid == 0) || !kwin_module->dockWindows().contains(wid))
	    m_Docks.remove(emb);
	else
	    m_Docks.next();
    }
    layoutDockingArea();
}
	
void DockAreaApplet::layoutDockingArea()
{
    if (m_Docks.count() == 0) {
        setMinimumSize(0, 0);
        hide();
	return;
    }
    int i;

    KXEmbed *emb;
    if (orientation() == Vertical) {
	want_height = m_Docks.count()*24 + 4;
	i = 0;
	for (emb = m_Docks.first(); emb != 0L; emb = m_Docks.next()) {
	    emb->move(width()/2-12, 2 + i*24);
	    i++;
	}
        setMinimumSize(3, want_height);
    } else {
	want_width = m_Docks.count()*24 + 4;
	i = 0;
	for (emb = m_Docks.first(); emb != 0L; emb = m_Docks.next()) {
	    emb->move(2 + i*24, height()/2-12);
	    i++;
        }
        setMinimumSize(want_width, 3);
    }
    updateGeometry();
    show();
}
