#include "userrectsel.h"
#include <qapplication.h>
#include <qpainter.h>
#include <X11/Xlib.h>

UserRectSel::UserRectSel( const QValueList<QRect>& rects, int start  )
    : QWidget( 0, 0, WStyle_Customize | WStyle_NoBorder ),
      rectangles( rects ),
      current ( start )
{
    setGeometry( -10, -10, 2, 2);
}

UserRectSel::~UserRectSel()
{
}

int UserRectSel::select()
{
    show();
    setMouseTracking( TRUE );
    XGrabServer( qt_xdisplay() );
    grabMouse();
    paintCurrent();
    qApp->enter_loop();
    paintCurrent();
    releaseMouse();
    XUngrabServer( qt_xdisplay() );
    qApp->syncX();
    return current;
}

void UserRectSel::mouseReleaseEvent( QMouseEvent * e)
{
    if ( e->button() == LeftButton ) {
	qApp->exit_loop();
    }
}

void UserRectSel::mouseMoveEvent( QMouseEvent * e)
{
    int nearest = current;
    int diff = -1;
    for (int i = 0; i < int(rectangles.count()); i++) {
	QRect r( rectangles[i] );
	int ndiff = (r.center().x() - e->globalPos().x() ) *  (r.center().x() - e->globalPos().x() )
		    +  (r.center().y() - e->globalPos().y() ) *  (r.center().y() - e->globalPos().y() );
	if ( r.contains( e->globalPos() ) )
	    ndiff = 0;
	if ( diff < 0 || ndiff < diff ) {
	    diff = ndiff;
	    nearest = i;
	}
    }
    if ( nearest != current ) {
	paintCurrent();
	current = nearest;
	paintCurrent();
    }
}

void UserRectSel::paintCurrent()
{
    QRect r( rectangles[current] );
    QWidget w(0, "", WType_Desktop | WPaintUnclipped );
    QPainter p ( &w );
    p.setPen( QPen( white, 5 ) );
    p.setRasterOp( XorROP );
    p.drawRect( r );
}


int UserRectSel::select( const QValueList<QRect>& rects, int start )
{
    UserRectSel sel( rects, start );
    return sel.select();
}

#include "userrectsel.moc"

