#include "clock.h"

#include <qtimer.h>
#include <qdatetime.h>
#include <qlayout.h>
#include <kglobal.h>
#include <kconfig.h>
#include <klocale.h>
#include <kapp.h>
#include <kcmdlineargs.h>
#include "kwinmodule.h"

static const char *description = I18N_NOOP("A Cool Clock for Kicker");

static const char *version = "v0.0.1";

extern "C"
{
    InternalApplet* init(QWidget *parent, KWinModule *)
    {
        return new ClockWidget(parent, "koolclockapplet");
    }
}

Clock::Clock(QWidget *parent, const char *name)
    : QLCDNumber(parent, name)
{
    setFrameStyle(NoFrame);

    KConfig *config = KGlobal::config();
    config->setGroup("koolclock");
    QString tmpStr = config->readEntry("Style", "Flat");
    if(tmpStr == "Flat")
        setSegmentStyle(QLCDNumber::Flat);
    else if(tmpStr == "Filled")
        setSegmentStyle(QLCDNumber::Filled);
    else
        setSegmentStyle(QLCDNumber::Outline);
    blink = config->readBoolEntry("Blink", false);
    ampm = config->readBoolEntry("AmPm", true);

    timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(slotTimer()));
    slotTimer();
    timer->start(1000);
}

void Clock::slotTimer()
{
    static bool colon = true;
    QString newStr;
    int h = QTime::currentTime().hour();
    if(ampm && h > 12)
        h -= 12;
    if(colon)
        newStr.sprintf("%02d:%02d", h, QTime::currentTime().minute());
    else
        newStr.sprintf("%02d %02d", h, QTime::currentTime().minute());

    if(newStr != timeStr){
        timeStr = newStr;
        display(timeStr);
    }
    if(blink)
        colon = !colon;
}

Date::Date(QWidget *parent, const char *name)
    : QLabel(parent, name)
{
    setAlignment(AlignCenter);
    QFont newFont(font());
    newFont.setPointSize(10);
    setFont(newFont);
    timer = new QTimer(this);

    connect(timer, SIGNAL(timeout()), this, SLOT(slotTimer()));
    slotTimer();
    timer->start(600000);
}

void Date::slotTimer()
{
    QDate date = QDate::currentDate();
    QString newStr = KGlobal::locale()->formatDate(QDate::currentDate(), true);
    if(newStr != dateStr){
        dateStr = newStr;
        setText(dateStr);
    }
}

ClockWidget::ClockWidget(QWidget *parent, const char *name)
    : InternalApplet(parent, name)
{
    layout = new QVBoxLayout(this);
    clock = new Clock(this);
    date = new Date(this);
}

void ClockWidget::setupGeometry(Orientation, int, int h)
{
    delete layout;
    if(h < 32)
        layout = new QHBoxLayout(this);
    else
        layout = new QVBoxLayout(this);

    layout->addWidget(clock);
    layout->addWidget(date);

}

int main(int argc, char **argv)
{
    KCmdLineArgs::init(argc, argv, "koolclock", description, version );
    KApplication app;
    ClockWidget *w = new ClockWidget;
    app.setMainWidget(w);
    w->resize(w->sizeHint());
    w->show();
    return(app.exec());
}


#include "clock.moc"

