#ifndef TASKBAR_H
#define TASKBAR_H

#include <qpushbutton.h>
#include <qlist.h>
#include <qpixmap.h>
#include "../../internalapplet.h"


class TaskButton: public QPushButton
{
    Q_OBJECT
public:
    TaskButton( WId w, QWidget * parent, const char *name = 0 );
    ~TaskButton();

    inline WId window() const;
    void setActiveButton();
    void refresh();
    bool onCurrentDesktop() const;
    QSizePolicy sizePolicy() const;
    QSize sizeHint() const;
    static void noActiveButton();

protected:
    enum WindowOperation {
        MaximizeOp = 100,
        IconifyOp,
        CloseOp
    };
    void drawButton(QPainter *p);
    void mouseReleaseEvent( QMouseEvent *e );

private slots:
    void toggled();
    void clientPopupAboutToShow();
    void desktopPopupAboutToShow();

    void clientPopupActivated(int);
    void sendToDesktop(int);

private:
    static TaskButton* activeButton;
    WId win;
    QPixmap pixmap;
    int desktop;
    bool sticky;

    QPopupMenu *popup;
    QPopupMenu *desk_popup;
};

class TaskbarApplet : public InternalApplet
{
    Q_OBJECT
public:
    TaskbarApplet(QWidget* parent, const char* name = 0);
    ~TaskbarApplet();
    void setupGeometry(Orientation orient, int w, int h);
    void updateLayout();
private slots:
    void windowAdd(WId);
    void windowRemove(WId);
    void windowChange(WId);
    void windowActivate(WId);

    void desktopChange(int);

private:
    QList<TaskButton> buttons;
    int taskbar_size;
    int lastH;
    TaskButton* findButton( WId w );
};


inline WId TaskButton::window() const
{
    return win;
}


#endif
