/*
 * (C) Daniel M. Duley (mosfet@kde.org) KDE artistic license.
 */

// menu classes

#ifndef __PANEL_MENUS_H
#define __PANEL_MENUS_H
#include <kpopmenu.h>
#include <qlist.h>
#include <qstringlist.h>
#include <qfileinfo.h>
#include <qtimer.h>
#include "clientmenu.h"
#include <qdir.h>

//class QDir; //conficting header hack
typedef QListIterator<QFileInfo>QFileInfoListIterator;


// Base class for file and applnk menus
class PanelMenu : public KPopupMenu
{
    Q_OBJECT
public:
    PanelMenu(const QString &startDir, QWidget *parent=0,
              const char *name=0);
    ~PanelMenu(){;}
    const QString& path(){return(startPath);}
    void setPath(const QString &p) { startPath = p; }
    bool initialized(){return(init);}
public slots:
    virtual void slotAboutToShow();
    virtual void initialize(bool reInit = false)=0;
protected slots:
    virtual void slotExec(int id)=0;
    void slotClear();
protected:
  virtual void hideEvent(QHideEvent *ev);
  bool init;
  int clearDelay;
  QString startPath;
  QTimer t;
};

// Both a file and applnk directory structure browser popupmenu
class PanelBrowserMenu : public PanelMenu
{
    Q_OBJECT
public:
    PanelBrowserMenu(QString startDir, bool expandFSTypes=false,
                     bool ignoreEmptyDirs=false, QWidget *parent=0,
                     const char *name=0,
                     int filter = QDir::Hidden | QDir::Dirs | QDir::Files,
                     int sorting = QDir::DirsFirst| QDir::Name);

    void append(const QPixmap &pixmap, const QString &title,
                const QString &filename);
    void append(const QPixmap &smallPixmap, const QPixmap &mediumPixmap,
		const QString &title,
		const QString &filename);
    void append(const QIconSet &iconSet, const QString &title,
		const QString &filename);
    void append(const QPixmap &pixmap, const QString &title,
                PanelBrowserMenu *subMenu);
    void append(const QPixmap &smallPixmap, const QPixmap &mediumPixmap,
		const QString &title,
		PanelBrowserMenu *subMenu);
    void append(const QIconSet &iconSet, const QString &title,
                PanelBrowserMenu *subMenu);

public slots:
    virtual void initialize(bool reInit = false);
protected slots:
    virtual void slotExec(int id);
protected:
  // for drag and drop 
  void mousePressEvent(QMouseEvent *);
  void mouseMoveEvent(QMouseEvent *);  
  QPoint m_pressedPos;
  
  
  QStringList fileList;
  QList<PanelBrowserMenu> subMenus;
  unsigned int max;
  bool merge;
  bool maxedOut, expandFS, ignoreEmpty;
  QDir::FilterSpec filterSpec;
  QDir::SortSpec sortSpec;
};

// The toplevel menu for the quickbrowser
class PanelQuickBrowser : public PanelMenu
{
    Q_OBJECT
public:
    PanelQuickBrowser(QWidget *parent=0, const char *name=0);
public slots:
    virtual void initialize(bool reInit = false);
protected slots:
    virtual void slotExec(int id);
protected:
    void parseBrowserApplnks(QFileInfoListIterator &it, const QString &title,
                             const QString &pixmap);
    QStringList fileList;
    unsigned int max;
};

// A exec menu with the Kicker special popupmenu items.
class PanelKMenu : public PanelBrowserMenu
{
    Q_OBJECT
public:
    PanelKMenu(QWidget *parent=0, const char *name=0);
    ~PanelKMenu(){;}

    virtual void initialize(bool reInit = false); // call this before manually adding menus
    void removeChildMenu(QPopupMenu *popup); // for removing client menus
protected slots:
    void slotLock();
    void slotLogout();
    void slotConfigure();
    void slotRestart();
    void slotSizeSmall();
    void slotSizeNormal();
    void slotSizeLarge();
    void slotContextHelp();
};

// Menu for the "Recent Documents". It mostly just re-initializes when shown.
class PanelRecentMenu : public PanelBrowserMenu
{
    Q_OBJECT
public:
    PanelRecentMenu(QWidget *parent=0, const char *name=0);
    virtual void initialize(bool reInit = false);
protected slots:
    void slotClearHistory();
};


// The button operations menu (usually right click)
class PanelButtonOpMenu : public KPopupMenu
{
public:
    enum OpButton{Move=1, Remove, Properties};
    PanelButtonOpMenu(QWidget *parent=0, const char *name=0,
                      bool removable=true);
};

// The directory dropped menu
class PanelDirDropMenu : public KPopupMenu
{
public:
    enum OpButton{Url=1, Browser};
    PanelDirDropMenu(QWidget *parent=0, const char *name=0);
};



#endif


