#include <panelbutton.h> // need to include before kwm.h due to conflicts
#include <kwm.h>
#undef KeyPress
#undef KeyRelease
#undef Bool
#include <qapplication.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qscrollview.h>
#include <kprocess.h>
#include <kglobal.h>
#include <kconfig.h>
#include <kiconloader.h>
#include "panel.h"
#include "panner.h"
#include "itemarea.h"
#include "appletarea.h"
#include "userrectsel.h"
#include "pluginmgr.h"
#include <math.h>
#include <kwinmodule.h>
#include <dcopclient.h>
#include <X11/Xlib.h>

template class QIntDict<void*>;

Panel *panel; // so everyone can easily get things like size;

KWinModule* kwin_module=0;

const int arrow = 12;

/*
  like QPushButton but without clever sizehint so it fits better in the panel's layout
*/
class PushButton : public QPushButton
{
public:
    PushButton( QWidget *parent, const char *name=0 )
        : QPushButton( parent, name )
        {}
    QSize sizeHint() const
        {
            return QWidget::sizeHint();
        }
    QSizePolicy sizePolicy() const
        {
            return QWidget::sizePolicy();
        }
};


Panel::Panel(Position position, int size )
    : QFrame( 0, "panel" )
{
    delete kwin_module;
    kwin_module = new KWinModule( this, TRUE  );
    panel = this;
    pluginManager = new KickerPluginManager;

    KWM::setWindowRegion( KWM::currentDesktop(), QApplication::desktop()->geometry() );
    KWM::setDecoration( winId(), KWM::noDecoration );
    KWM::setSticky( winId(), TRUE );
    setCaption("Kicker");
    KConfig *config = KGlobal::config();
    config->setGroup("panel");
    smallExtent = config->readNumEntry("SmallSize", 32);
    normalExtent = config->readNumEntry("NormalSize", 48);
    largeExtent = config->readNumEntry("LargeSize", 56);
    sizeState = (UserSize)config->readNumEntry("SizeMode", Large);
    QStringList splitter_lst = config->readListEntry("Splitter");

    panelsize = extentForSize(sizeState);

    pos = (Position)config->readNumEntry("Position", Bottom);

    qDebug("******want to be %d %d", initialGeometry().width(), initialGeometry().height() );
    setGeometry( initialGeometry() );

    installEventFilter( this );

    setFrameStyle( WinPanel  | Raised );

    layout = new QBoxLayout( this, isHorizontal()?QBoxLayout::LeftToRight:QBoxLayout::TopToBottom,
                             frameWidth(), 2 );
    layout->setResizeMode( QLayout::FreeResize );

    // left arrow
    leftArrow = new PushButton( this );
    connect( leftArrow, SIGNAL( clicked() ), this, SLOT( hideLeft() ) );
    if ( isHorizontal() )
        leftArrow->setFixedSize( arrow, contentsRect().height() );
    else
        leftArrow->setFixedSize( contentsRect().width(), arrow );

    layout->addWidget( leftArrow );

    // the splitter
    splitter = new QSplitter( isHorizontal()?Horizontal:Vertical, this );
    splitter->setOpaqueResize( TRUE );
    layout->addWidget( splitter, 1 );
    if ( isHorizontal() )
        splitter->setFixedHeight( contentsRect().height() );
    else
        splitter->setFixedWidth( contentsRect().width() );

    // restore Splitter positions
    QValueList<int> sp_sizes;
    for(QStringList::ConstIterator it = splitter_lst.begin(); it != splitter_lst.end(); it++)
        sp_sizes += (*it).toInt();

    // splitter contains the applet area
    itemArea= new ItemArea("buttons", orientation(), buttonSize(), splitter );
    itemArea->viewport()->installEventFilter( this );
    itemArea->setFrameStyle( QFrame::NoFrame );
    itemArea->setPosition(pos);
    
    //applets
    appletArea = new AppletArea( orientation(), buttonSize(), splitter );
    appletArea->setFrameStyle( config->readBoolEntry("AppletFrame", true) ?
                               QFrame::Sunken|QFrame::Panel : QFrame::NoFrame );
    //CT or I can simply parent it to the panel and gets a fixed size through
    //  the layout (have to add it to the layout too
    appletArea->viewport()->installEventFilter( this );

    rightArrow = new PushButton( this );
    connect( rightArrow, SIGNAL( clicked() ), this, SLOT( hideRight() ) );
    if ( isHorizontal() )
        rightArrow->setFixedSize( arrow, contentsRect().height() );
    else
        rightArrow->setFixedSize( contentsRect().width(), arrow );
    layout->addWidget( rightArrow );

    splitter->installEventFilter( this );
    splitter->setSizes(sp_sizes);
    block_user_input = FALSE;

}

QValueList<int> Panel::sizes() {
  return splitter->sizes();
}

void Panel::setSizes(QValueList<int> sizes) {
  splitter->setSizes(sizes);
}

int Panel::extentForSize(UserSize state)
{
    switch(state)
    {
    case Tiny:
        return smallExtent;
    case Normal:
        return normalExtent;
    case Large:
    default:
        return largeExtent;
    }
}

void Panel::setUserSize(UserSize state)
{
    sizeState = state;
    resetLayout(pos, panelsize=extentForSize(state));
    writeConfig();
} 

void Panel::writeConfig()
{
    KConfig *config = KGlobal::config();
    config->setGroup("panel");
    config->writeEntry("Position", static_cast<int>(pos));
    config->writeEntry("SizeMode", (int)sizeState);
    QStringList lst;
    QValueList<int> sp_sizes = splitter->sizes();
    for(QValueList<int>::ConstIterator it = sp_sizes.begin(); it != sp_sizes.end(); it++)
        lst += QString().setNum(*it);

    config->writeEntry("Splitter", lst);
    config->sync();
}

void Panel::resetLayout(Position position, int size)
{
    // I don't think we have to do this anymore (mosfet)
    // delete kwin_module;
    // kwin_module = new KWinModule( this, TRUE  );
    debug("resetLayout");
    panelsize = size;
    pos = position;

    writeConfig();

    delete layout;

    setGeometry( initialGeometry() );
    layout = new QBoxLayout( this,
         isHorizontal()?QBoxLayout::LeftToRight:QBoxLayout::TopToBottom,
                             frameWidth(), 2);
    layout->setResizeMode( QLayout::FreeResize );

    if ( isHorizontal() )
        leftArrow->setFixedSize( arrow, contentsRect().height() );
    else
	leftArrow->setFixedSize( contentsRect().width(), arrow );
    layout->addWidget( leftArrow );

    splitter->setOrientation(isHorizontal()?Horizontal:Vertical);
    if ( isHorizontal() )
        splitter->setFixedHeight( contentsRect().height() );
    else
        splitter->setFixedWidth( contentsRect().width() );
    layout->addWidget( splitter, 1 );

    itemArea->setButtonSize(buttonSize());
    itemArea->setPosition(pos);
    itemArea->setOrientation(orientation());
    appletArea->setOrientation(orientation());

    if ( isHorizontal() )
        rightArrow->setFixedSize( arrow, contentsRect().height() );
    else
        rightArrow->setFixedSize( contentsRect().width(), arrow );

    layout->addWidget( rightArrow );
}

Panel::~Panel()
{
    writeConfig();
    delete pluginManager;
}

QRect Panel::initialGeometry()
{
    QRect r( QApplication::desktop()->geometry() );
    switch( pos ) {
    case Left:
        return QRect(  r.left(), r.top(), panelsize, r.height() );
    case Right:
        return QRect(  r.right() - panelsize + 1, r.top(), panelsize, r.height() );
    case Top:
        return QRect(  r.left(), r.top(), r.width(), panelsize );
    case Bottom:
    default:
        return QRect(  r.left(), r.bottom() - panelsize + 1, r.width(), panelsize );
    }
}

void Panel::resizeEvent( QResizeEvent* e )
{
    QFrame::resizeEvent( e );
//     const int arrow = 12;
//     if ( isHorizontal() ) {
// 	leftArrow->setGeometry( 0, 0, arrow, height() );
// 	rightArrow->setGeometry( width() - arrow, 0, arrow, height() );
// 	splitter->setGeometry( leftArrow->geometry().right()+1, contentsRect().top(),
// 			       width() - 2*arrow, contentsRect().height() );
//     } else {
// 	leftArrow->setGeometry( 0, 0, width(), arrow );
// 	rightArrow->setGeometry( 0, height() - arrow, width(), arrow );
// 	splitter->setGeometry( contentsRect().left(), leftArrow->geometry().bottom()+1,
// 			       contentsRect().width(), height() - 2*arrow);
//     }
}

const int hide_show_animation = 50;
/* 1 is the initial speed, hide_show_animation is the top speed. */
#define PANEL_SPEED(x, c) (int)((1.0-2.0*fabs((x)-(c)/2.0)/c)*hide_show_animation+1.0)

void Panel::hideLeft()
{
    blockUserInput( TRUE );
    QRect geom( initialGeometry() );

    if ( geom != geometry() ) {
        // show from right first
        if ( isVertical() ) {
            int strut = leftArrow->geometry().bottom();
            for (int i = geom.height()-strut; i>0;i-=PANEL_SPEED(i,geom.height()) ){
                move(geom.x(), geom.y()+i);
                qApp->syncX();
                qApp->processEvents();
            }
        } else {
            int strut = leftArrow->geometry().right();
            for (int i = geom.width()-strut; i>0;i-=PANEL_SPEED(i,geom.width()) ){
                move(geom.x()+i, geom.y());
                qApp->syncX();
                qApp->processEvents();
            }
        }

        move(geom.x(), geom.y());
    } else {
        if ( isVertical() ) {
            int strut = leftArrow->geometry().bottom();
            for (int i = 0;
                 i < geom.height() - strut;
                 i += PANEL_SPEED(i,geom.height())){
                move(geom.x(), geom.y()-i);
                qApp->syncX();
                qApp->processEvents();
            }
        } else {
            int strut = leftArrow->geometry().right();
            for (int i = 0;
                 i < geom.width() - strut;
                 i += PANEL_SPEED(i,geom.width())){
                move(geom.x()-i, geom.y());
                qApp->syncX();
                qApp->processEvents();
            }
        }
    }
    blockUserInput( FALSE );
}

void Panel::hideRight()
{
    blockUserInput( TRUE );
    QRect geom( initialGeometry() );

    if ( geom != geometry() ) {
        // show from left first
        if ( isVertical() ) {
            int strut = leftArrow->geometry().bottom();
            for (int i = geom.height()-strut; i>0;i-=PANEL_SPEED(i,geom.height()) ){
                move(geom.x(), geom.y()-i);
                qApp->syncX();
                qApp->processEvents();
            }
        } else {
            int strut = leftArrow->geometry().right();
            for (int i = geom.width()-strut; i>0;i-=PANEL_SPEED(i,geom.width()) ){
                move(geom.x()-i, geom.y());
                qApp->syncX();
                qApp->processEvents();
            }
        }

        move(geom.x(), geom.y());
    } else {
        if ( isVertical() ) {
            int strut = leftArrow->geometry().bottom();
            for (int i = 0;
                 i < geom.height() - strut;
                 i += PANEL_SPEED(i,geom.height())){
                move(geom.x(), geom.y()+i);
                qApp->syncX();
                qApp->processEvents();
            }
        } else {
            int strut = leftArrow->geometry().right();
            for (int i = 0;
                 i < geom.width() - strut;
                 i += PANEL_SPEED(i,geom.width())){
                move(geom.x()+i, geom.y());
                qApp->syncX();
                qApp->processEvents();
            }
        }
    }
    blockUserInput( FALSE );
}

bool Panel::eventFilter( QObject* , QEvent * e)
{
    if ( block_user_input ) {
        switch ( e->type() ) {
        case QEvent::MouseButtonPress:
        case QEvent::MouseButtonRelease:
        case QEvent::MouseButtonDblClick:
        case QEvent::MouseMove:
        case QEvent::KeyPress:
        case QEvent::KeyRelease:
            return TRUE; // ignore;
        default:
	    break;
        }
    }

    switch ( e->type() ) {
    case QEvent::MouseButtonPress:
	{
	    QMouseEvent* me = (QMouseEvent*) e;
	    if ( me->button() == LeftButton )
            last_lmb_press = me->globalPos();
	}
	break;
    case QEvent::MouseMove:
	{
	    QMouseEvent* me = (QMouseEvent*) e;
	    if (  (me->state() & LeftButton) == LeftButton ) {
            QPoint p ( me->pos() - last_lmb_press );
            if ( !geometry().contains(me->globalPos()) &&
                (QABS( me->pos().x() - p.x()  ) > 5 || QABS( me->pos().y() - p.y() ) > 5 ) ) {
                emit moveMe();
                return TRUE;
            }
	    }
	}
	break;
    default:
        break;
    }
    return FALSE;
}

void Panel::blockUserInput( bool block )
{
    if ( block == block_user_input )
        return;
    if ( block )
        qApp->installEventFilter( this );
    else
        qApp->removeEventFilter( this );

    block_user_input = block;
}

void Panel::moveMe()
{
    hide();
    QApplication::syncX();
    QValueList<QRect> rects;
    QRect r( QApplication::desktop()->geometry() );
    rects.append( QRect(  r.left(), r.top(), panelsize, r.height() ) );
    rects.append( QRect(  r.right() - panelsize + 1, r.top(), panelsize, r.height() ) );
    rects.append( QRect(  r.left(), r.top(), r.width(), panelsize ) );
    rects.append( QRect(  r.left(), r.bottom() - panelsize + 1, r.width(), panelsize ) );
    Position newpos = (Position) UserRectSel::select(rects, pos);
    if ( newpos != pos ) {
        pos = newpos;
        resetLayout(pos, panelsize);
    }
    show();
}

