#ifndef PANEL_H
#define PANEL_H
#include <qwidget.h>
#include <qframe.h>
#include <qsplitter.h>
#include <qlayout.h>
class QPushButton;
class Panner;
class ItemArea;
class AppletArea;
class KickerPluginManager;

// ItemArea and PanelButton use this as well
enum Position { Left = 0, Right, Top, Bottom }; 

class Panel : public QFrame
{
    Q_OBJECT

public:
    enum UserSize {Tiny=0, Normal, Large};

    Panel( Position position, int size );
    ~Panel();

    Orientation orientation() const;
    ItemArea *btChildArea(){return(itemArea);}
    AppletArea *apChildArea(){return(appletArea);}
    bool isHorizontal() const;
    bool isVertical() const;
    int panelSize() const;
    int buttonSize() const;
    KickerPluginManager* pluginMgr(){return(pluginManager);}
    Position position() const;

    bool eventFilter( QObject *, QEvent * );

    void resetLayout(Position position, int size);
    int extentForSize(UserSize state);
    UserSize userSize(){return(sizeState);}
    void setUserSize(UserSize state);
    void moveMe();

    QValueList<int> sizes();
    void setSizes(QValueList<int>);

protected:
    void resizeEvent( QResizeEvent* );

private slots:
    void hideLeft();
    void hideRight();

private:
    QBoxLayout *layout;
    UserSize sizeState;
    int panelsize; // width or height, depending on the orientation
    int smallExtent, normalExtent, largeExtent; // user defined extent sizes
    QPoint last_lmb_press;
    Position pos;

    QRect initialGeometry();


    QPushButton* leftArrow;
    QPushButton* rightArrow;
    QSplitter* splitter;

    ItemArea* itemArea;
    AppletArea* appletArea;
    bool block_user_input;
    KickerPluginManager *pluginManager;

    void blockUserInput( bool block );
    void writeConfig();
};

inline bool Panel::isHorizontal() const
{
    return pos == Top || pos == Bottom;
}
inline bool Panel::isVertical() const
{
    return !isHorizontal();
}
inline Qt::Orientation Panel::orientation() const
{
    return isHorizontal()?Horizontal:Vertical;
}

inline int Panel::panelSize() const
{
    return panelsize;
}

inline int Panel::buttonSize() const
{
    return panelsize - (2 * frameWidth());
}

inline Position Panel::position() const
{
    return pos;
}

#endif
