#include "pluginmgr.h"
#include "panel.h"
#include <kwinmodule.h>
#include <ksimpleconfig.h>
#include <kglobal.h>
#include <kstddirs.h>

extern KWinModule *kwin_module;

InternalApplet* KickerPluginManager::loadPlugin(const QString &applnk,
                                                QWidget *parent)
{
    static bool dlregistered = false;
    QString libStr, applnkPathStr;
    applnkPathStr = KGlobal::dirs()->findResource("appdata", applnk);
    KSimpleConfig config(applnkPathStr);
    config.setDesktopGroup();
    libStr = config.readEntry("X-KDE-Library", "");
    libStr += ".la";
    libStr = KGlobal::dirs()->findResource("lib", libStr);
    if(!libStr){
        warning("Kicker: cannot find kicker internal applet");
        return(NULL);
    }

    if(!dlregistered){
        dlregistered = true;
        lt_dlinit();
    }

    lt_dlhandle handle = lt_dlopen(libStr.latin1());
    if(!handle){
        warning("Kicker: cannot open kicker internal applet: %s",
                libStr.latin1());
        return(NULL);
    }

    lt_ptr_t init_func = lt_dlsym(handle, "init");
    if(!init_func){
        warning("Kicker: %s is not a kicker internal applet!",
                libStr.latin1());
        lt_dlclose(handle);
        return(NULL);
    }
    InternalApplet* (*init_ptr)(QWidget *, KWinModule *);
    init_ptr = (InternalApplet* (*)(QWidget *, KWinModule *))init_func;
    InternalApplet *w = init_ptr(parent, kwin_module);
    lt_dlhandle *tmp = new lt_dlhandle;
    *tmp = handle;
    handleDict.insert((long)w, tmp);
	return(w);
}

bool KickerPluginManager::removePlugin(QWidget *w)
{
    lt_dlhandle *handle = handleDict.find((long)w);
    if(!handle){
        warning("Kicker: Cannot find handle to remove internal applet!");
        return(false);
    }
    handleDict.remove((long)w);
    delete w;
    if(!lt_dlclose(*handle)){
        warning("Kicker: Unable to unload internal applet!");
        return(false);
    }
    return(true);
}




        
